using FitConnect.Models;
using FitConnect.Models.Api.Metadata;
using FitConnect.Models.v1.Api;
using Attachment = FitConnect.Models.Attachment;

namespace FitConnect.Interfaces.Subscriber;

public interface IReceivedSubmission {
    internal Submission? Submission { get; }
    Guid? SubmissionId => Submission?.SubmissionId;
    Guid? CaseId => Submission?.CaseId;
    MetadataDto? Metadata => Submission?.Metadata;
    Uri? DataSchemaUri { get; }
    Guid? DestinationId { get; }

    /// <summary>
    ///     Returns the data (Fachdaten) of the submission
    /// </summary>
    /// <returns></returns>
    public string? GetData() => Submission?.Data;

    /// <summary>
    ///     Loads the <see cref="Models.Attachment" />s for the given <see cref="Submission" />.
    /// </summary>
    /// <returns>Enumerable of the attachments</returns>
    public Task<IEnumerable<Attachment>> GetAttachmentsAsync();

    /// <summary>
    ///     Accept submission and delete it from the server
    /// </summary>
    public Task AcceptSubmissionAsync(params Problems[] problems);

    /// <summary>
    ///     Rejects the submission
    /// </summary>
    /// <param name="problems">Reasons for the rejection</param>
    public Task RejectSubmissionAsync(params Problems[] problems);

    /// <summary>
    /// Returns the mime type of the submitted data
    /// </summary>
    /// <returns></returns>
    string GetDataMimeType();
}
