using FitConnect.Models;
using FitConnect.Models.Api.Metadata;
using FitConnect.Models.Api.Set;

namespace FitConnect;

public interface ISendableSubmission {
    Guid DestinationId { get; }
    string? ServiceName { get; }
    string? ServiceIdentifier { get; }
    string? Data { get; }
    Uri? SchemaUri { get; }
}

public class SendableSubmission: ISendableSubmission {
    protected SendableSubmission() {
        DestinationId = new Guid();
    }

    internal SendableSubmission(Guid destinationId) {
        DestinationId = destinationId;
    }

    public static SubmissionBuilder Builder() => new SubmissionBuilder();

    public Guid DestinationId { get; set; }
    public string? ServiceName { get; protected set; }
    public string? ServiceIdentifier { get; protected set; }
    public List<Attachment>? Attachments { get; internal set; }
    public string? Data { get; internal set; }
    public string? DataMimeType { get; internal set; }
    public Uri? SchemaUri { get; internal set; }
    public PaymentInformation? PaymentInformation { get; protected set; }
    public ReplyChannel? ReplyChannel { get; protected set; }
    public AuthenticationInformation[]? AuthenticationInformation { get; protected set; }

    public Submission ToSubmission() => new() {
        DestinationId = DestinationId,
        ServiceName = ServiceName,
        ServiceIdentifier = ServiceIdentifier,
        Attachments = Attachments,
        Data = Data,
        DataMimeType = DataMimeType,
        PaymentInformation = PaymentInformation,
        ReplyChannel = ReplyChannel,
        AuthenticationInformation = AuthenticationInformation
    };

}

public class SendableEncryptedSubmission : ISendableSubmission {
    internal SendableEncryptedSubmission() {
        
    }
    public string? Metadata { get; set; }

    public static EncryptedSubmissionBuilder Builder() {
        return new EncryptedSubmissionBuilder();
    }

    public string? Metadata { get; internal set; }
    public Dictionary<Guid, string>? Attachments { get; internal set; }
    public Guid DestinationId { get; set; }
    public string? ServiceName { get; internal set; }
    public string? ServiceIdentifier { get; internal set; }

    /// <summary>
    ///     The encrypted data.
    /// </summary>
    public string? Data { get; internal set; }

    public Uri? SchemaUri { get; set; }
}
