// <auto-generated />
//
// To parse this JSON data, add NuGet 'Newtonsoft.Json' then do:
//
//    using FitConnect;
//
//    var metadata = Metadata.FromJson(jsonString);

using System.Runtime.Serialization;
using FitConnect.Models.Api.Set;

namespace FitConnect.Models.Api.Metadata
{
    using System;
    using System.Collections.Generic;

    using System.Globalization;
    using Newtonsoft.Json;
    using Newtonsoft.Json.Converters;

    public class MetadataDto
    {
        [JsonProperty("$schema")]
        public Uri SchemaUri { get; set; }
        
        /// <summary>
        /// Eine Struktur, um zusätzliche Informationen zu hinterlegen
        /// </summary>
        [JsonProperty("additionalReferenceInfo", NullValueHandling = NullValueHandling.Ignore)]
        public AdditionalReferenceInfo AdditionalReferenceInfo { get; set; }

        /// <summary>
        /// Eine Liste aller Identifikationsnachweise der Einreichung.
        /// </summary>
        [JsonProperty("authenticationInformation", NullValueHandling = NullValueHandling.Ignore)]
        public List<AuthenticationInformation> AuthenticationInformation { get; set; }

        /// <summary>
        /// Beschreibt die Struktur der zusätzlichen Inhalte der Einreichung, wie Anlagen oder
        /// Fachdaten.
        /// </summary>
        [JsonProperty("contentStructure")]
        public ContentStructure ContentStructure { get; set; }

        /// <summary>
        /// Dieses Objekt enthält die Informationen vom Bezahldienst.
        /// </summary>
        [JsonProperty("paymentInformation", NullValueHandling = NullValueHandling.Ignore)]
        public PaymentInformation PaymentInformation { get; set; }

        /// <summary>
        /// Beschreibung der Art der Verwaltungsleistung. Eine Verwaltungsleistung sollte immer mit
        /// einer LeiKa-Id beschrieben werden. Ist für die gegebene Verwaltungsleistung keine
        /// LeiKa-Id vorhanden, kann die Verwaltungsleistung übergangsweise über die Angabe einer
        /// anderen eindeutigen Schema-URN beschrieben werden.
        /// </summary>
        [JsonProperty("publicServiceType", NullValueHandling = NullValueHandling.Ignore)]
        internal Verwaltungsleistung PublicServiceType {  get;  set; }

        [JsonProperty("replyChannel", NullValueHandling = NullValueHandling.Ignore)]
        public ReplyChannel ReplyChannel { get; set; }
        
        public override string ToString() => JsonConvert.SerializeObject(this);
    }

    /// <summary>
    /// Eine Struktur, um zusätzliche Informationen zu hinterlegen
    /// </summary>
    public partial class AdditionalReferenceInfo
    {
        /// <summary>
        /// Das Datum der Antragstellung. Das Datum muss nicht zwingend identisch mit dem Datum der
        /// Einreichung des Antrags über FIT-Connect sein.
        /// </summary>
        [JsonProperty("applicationDate", NullValueHandling = NullValueHandling.Ignore)]
        public DateTimeOffset? ApplicationDate { get; set; }

        /// <summary>
        /// Eine Referenz zum Vorgang im sendenden System, um bei Problemen und Rückfragen
        /// auÃŸerhalb von FIT-Connect den Vorgang im dortigen System schneller zu identifizieren.
        /// </summary>
        [JsonProperty("senderReference", NullValueHandling = NullValueHandling.Ignore)]
        public string SenderReference { get; set; }
    }

    /// <summary>
    /// Beschreibt die Struktur der zusätzlichen Inhalte der Einreichung, wie Anlagen oder
    /// Fachdaten.
    /// </summary>
    public partial class ContentStructure
    {
        [JsonProperty("attachments")]
        public List<AttachmentMetadata> Attachments { get; set; }

        /// <summary>
        /// Definiert das Schema und die Signatur(-art), die für die Fachdaten verwendet werden.
        /// </summary>
        [JsonProperty("data", NullValueHandling = NullValueHandling.Ignore)]
        public Data Data { get; set; }
    }

    /// <summary>
    /// Eine in der Einreichung enthaltene Anlage.
    /// </summary>
    public partial class AttachmentMetadata
    {
        private string _filename;

        /// <summary>
        /// Innerhalb einer Einreichung eindeutige Id der Anlage im Format einer UUIDv4.
        /// </summary>
        [JsonProperty("attachmentId")]
        public Guid AttachmentId { get; internal set; }

        /// <summary>
        /// Optionale Beschreibung der Anlage
        /// </summary>
        [JsonProperty("description", NullValueHandling = NullValueHandling.Ignore)]
        public string Description { get; internal set; }

        /// <summary>
        /// Ursprünglicher Dateiname bei Erzeugung oder Upload
        /// </summary>
        [JsonProperty("filename", NullValueHandling = NullValueHandling.Ignore)]
        public string Filename {
            get => _filename;
            internal set => _filename = Path.GetFileName(value);
        }

        /// <summary>
        /// Der Hashwert der unverschlüsselten Anlage. Die Angabe des Hashwertes dient der
        /// Integritätssicherung des Gesamtantrags und schützt vor einem Austausch der Anlage durch
        /// Systeme zwischen Sender und Subscriber (z.B. dem Zustelldienst).
        /// </summary>
        [JsonProperty("hash")]
        public virtual AttachmentHash Hash { get; internal  set; }

        /// <summary>
        /// Internet Media Type gemäÃŸ RFC 2045, z. B. application/pdf.
        /// </summary>
        [JsonProperty("mimeType")]
        public string MimeType { get; internal set; }

        /// <summary>
        /// Zweck/Art der Anlage
        /// - form: Automatisch generierte PDF-Repräsentation des vollständigen Antragsformulars
        /// - attachment: Anlage, die von einem Bürger hochgeladen wurde
        /// - report: Vom Onlinedienst, nachträglich erzeugte Unterlage
        /// </summary>
        [JsonProperty("purpose")]
        public string Purpose { get; internal set; }

        [JsonProperty("signature", NullValueHandling = NullValueHandling.Ignore)]
        public AttachmentSignature Signature { get; protected set; }
    }

    /// <summary>
    /// Der Hashwert der unverschlüsselten Anlage. Die Angabe des Hashwertes dient der
    /// Integritätssicherung des Gesamtantrags und schützt vor einem Austausch der Anlage durch
    /// Systeme zwischen Sender und Subscriber (z.B. dem Zustelldienst).
    /// </summary>
    public partial class AttachmentHash
    {
        /// <summary>
        /// Der Hex-kodierte Hashwert gemäÃŸ des angegebenen Algorithmus.
        /// </summary>
        [JsonProperty("content")]
        public string Content { get; internal set; }

        /// <summary>
        /// Der verwendete Hash-Algorithmus. Derzeit ist nur `sha512` erlaubt.
        /// </summary>
        [JsonProperty("type")]
        public string Type { get; internal set; }
    }

    /// <summary>
    /// Der Hashwert der unverschlüsselten Fachdaten. Die Angabe des Hashwertes dient der
    /// Integritätssicherung des Gesamtantrags und schützt vor einem Austausch der Fachdaten
    /// durch Systeme zwischen Sender und Subscriber (z.B. dem Zustelldienst).
    /// </summary>
    public partial class DataHash
    {
        /// <summary>
        /// Der Hex-kodierte Hashwert gemäÃŸ des angegebenen Algorithmus.
        /// </summary>
        [JsonProperty("content")]
        public string Content { get; internal set; }

        /// <summary>
        /// Der verwendete Hash-Algorithmus. Derzeit ist nur `sha512` erlaubt.
        /// </summary>
        [JsonProperty("type")]
        public string Type { get; internal set; }
    }

    /// <summary>
    /// Beschreibt das Signaturformt und Profile
    /// </summary>
    public partial class DataSignature
    {
        /// <summary>
        /// Hier wird die Signatur im Falle einer Detached-Signatur als Base64- oder
        /// Base64Url-kodierte Zeichenkette hinterlegt. Eine Base64Url-Kodierung kommt nur bei
        /// Einsatz von JSON Web Signatures (JWS / JAdES) zum Einsatz.
        /// </summary>
        [JsonProperty("content", NullValueHandling = NullValueHandling.Ignore)]
        public string Content { get; internal set; }

        /// <summary>
        /// Beschreibt, ob die Signatur als seperate (detached) Signatur (`true`) oder als Teil des
        /// Fachdatensatzes bzw. der Anlage  (`false`) übertragen wird. Wenn der Wert `true` ist,
        /// dann wird die Signatur Base64- oder Base64Url-kodiert im Feld `content` übertragen.
        /// </summary>
        [JsonProperty("detachedSignature")]
        public bool DetachedSignature { get; internal set; }

        /// <summary>
        /// Referenziert ein eindeutiges Profil einer AdES (advanced electronic signature/seal)
        /// gemäÃŸ eIDAS-Verordnung über eine URI gemäÃŸ [ETSI TS 119
        /// 192](https://www.etsi.org/deliver/etsi_ts/119100_119199/119192/01.01.01_60/ts_119192v010101p.pdf).
        ///
        /// Für die Details zur Verwendung und Validierung von Profilen siehe auch
        /// https://ec.europa.eu/cefdigital/DSS/webapp-demo/doc/dss-documentation.html#_signatures_profile_simplification
        /// </summary>
        [JsonProperty("eidasAdesProfile", NullValueHandling = NullValueHandling.Ignore)]
        public EidasAdesProfile? EidasAdesProfile { get; internal set; }

        /// <summary>
        /// Beschreibt, welches Signaturformat die genutzte Signatur / das genutzte Siegel nutzt.
        /// Aktuell wird die Hinterlegung folgender Signaturformate unterstützt: CMS = Cryptographic
        /// Message Syntax, Asic = Associated Signature Containers, PDF = PDF Signatur, XML =
        /// XML-Signature, JSON = JSON Web Signature.
        /// </summary>
        [JsonProperty("signatureFormat")]
        public SignatureFormat SignatureFormat { get; internal set; }
    }

    /// <summary>
    /// Referenz auf ein Schema, das die Struktur der Fachdaten einer Einreichung beschreibt.
    /// </summary>
    public partial class Fachdatenschema
    {
        /// <summary>
        /// Mimetype (z.B. application/json oder application/xml) des referenzierten Schemas (z.B.
        /// XSD- oder JSON-Schema).
        /// </summary>
        [JsonProperty("mimeType")]
        public string MimeType { get; internal set; }

        /// <summary>
        /// URI des Fachschemas. Wird hier eine URL verwendet, sollte das Schema unter der
        /// angegebenen URL abrufbar sein. Eine Verfügbarkeit des Schemas unter der angegebenen URL
        /// darf jedoch nicht vorausgesetzt werden.
        /// </summary>
        [JsonProperty("schemaUri")]
        public Uri SchemaUri { get; internal set; }
    }

    /// <summary>
    /// Beschreibung der Art der Verwaltungsleistung. Eine Verwaltungsleistung sollte immer mit
    /// einer LeiKa-Id beschrieben werden. Ist für die gegebene Verwaltungsleistung keine
    /// LeiKa-Id vorhanden, kann die Verwaltungsleistung übergangsweise über die Angabe einer
    /// anderen eindeutigen Schema-URN beschrieben werden.
    /// </summary>
    public partial class Verwaltungsleistung
    {
        /// <summary>
        /// (Kurz-)Beschreibung der Verwaltungsleistung
        /// </summary>
        [JsonProperty("description", NullValueHandling = NullValueHandling.Ignore)]
        public string Description { get; internal set; }

        /// <summary>
        /// URN einer Leistung. Im Falle einer Leistung aus dem Leistungskatalog sollte hier
        /// `urn:de:fim:leika:leistung:` vorangestellt werden.
        /// </summary>
        [JsonProperty("identifier")]
        
        public string Identifier { get; internal set; }

        /// <summary>
        /// Name/Bezeichnung der Verwaltungsleistung
        /// </summary>
        [JsonProperty("name", NullValueHandling = NullValueHandling.Ignore)]
        public string Name { get; internal set; }
    }


    /// <summary>
    /// Definiert die Art des Identifikationsnachweises.
    /// </summary>
    public enum AuthenticationInformationType {
        [EnumMember(Value = "identificationReport")]
        IdentificationReport
    };

    /// <summary>
    /// Der verwendete Hash-Algorithmus. Derzeit ist nur `sha512` erlaubt.
    /// </summary>
    public enum HashType { Sha512 };

    /// <summary>
    /// Zweck/Art der Anlage
    /// - form: Automatisch generierte PDF-Repräsentation des vollständigen Antragsformulars
    /// - attachment: Anlage, die von einem Bürger hochgeladen wurde
    /// - report: Vom Onlinedienst, nachträglich erzeugte Unterlage
    /// </summary>
    public enum Purpose { Attachment, Form, Report };

    /// <summary>
    /// Referenziert ein eindeutiges Profil einer AdES (advanced electronic signature/seal)
    /// gemäÃŸ eIDAS-Verordnung über eine URI gemäÃŸ [ETSI TS 119
    /// 192](https://www.etsi.org/deliver/etsi_ts/119100_119199/119192/01.01.01_60/ts_119192v010101p.pdf).
    ///
    /// Für die Details zur Verwendung und Validierung von Profilen siehe auch
    /// https://ec.europa.eu/cefdigital/DSS/webapp-demo/doc/dss-documentation.html#_signatures_profile_simplification
    /// </summary>
    public enum EidasAdesProfile { HttpUriEtsiOrgAdes191X2LevelBaselineBB, HttpUriEtsiOrgAdes191X2LevelBaselineBLt, HttpUriEtsiOrgAdes191X2LevelBaselineBLta, HttpUriEtsiOrgAdes191X2LevelBaselineBT };

    /// <summary>
    /// Beschreibt, welches Signaturformat die genutzte Signatur / das genutzte Siegel nutzt.
    /// Aktuell wird die Hinterlegung folgender Signaturformate unterstützt: CMS = Cryptographic
    /// Message Syntax, Asic = Associated Signature Containers, PDF = PDF Signatur, XML =
    /// XML-Signature, JSON = JSON Web Signature.
    /// </summary>
    public enum SignatureFormat { Asic, Cms, Json, Pdf, Xml };
    

    
}
