// <auto-generated />
//
// To parse this JSON data, add NuGet 'Newtonsoft.Json' then do:
//
//    using FitConnect;
//
//    var metadata = Metadata.FromJson(jsonString);

using System.Runtime.Serialization;
using FitConnect.Models.Api.Metadata;

namespace FitConnect.Models.Api.Set
{
    using System;
    using System.Collections.Generic;

    using System.Globalization;
    using Newtonsoft.Json;
    using Newtonsoft.Json.Converters;


    /// <summary>
    /// Eine in der Einreichung enthaltene Anlage.
    /// </summary>
    public partial class AttachmentDto
    {
        /// <summary>
        /// Innerhalb einer Einreichung eindeutige Id der Anlage im Format einer UUIDv4.
        /// </summary>
        [JsonProperty("attachmentId")]
        public Guid AttachmentId { get; set; }

        /// <summary>
        /// Optionale Beschreibung der Anlage
        /// </summary>
        [JsonProperty("description", NullValueHandling = NullValueHandling.Ignore)]
        public string Description { get; set; }

        /// <summary>
        /// Ursprünglicher Dateiname bei Erzeugung oder Upload
        /// </summary>
        [JsonProperty("filename", NullValueHandling = NullValueHandling.Ignore)]
        public string Filename { get; set; }

        /// <summary>
        /// Der Hashwert der unverschlüsselten Anlage. Die Angabe des Hashwertes dient der
        /// Integritätssicherung des Gesamtantrags und schützt vor einem Austausch der Anlage durch
        /// Systeme zwischen Sender und Subscriber (z.B. dem Zustelldienst).
        /// </summary>
        [JsonProperty("hash")]
        public AttachmentHash Hash { get; set; }

        /// <summary>
        /// Internet Media Type gemäß RFC 2045, z. B. application/pdf.
        /// </summary>
        [JsonProperty("mimeType")]
        public string MimeType { get; set; }

        /// <summary>
        /// Zweck/Art der Anlage
        /// - form: Automatisch generierte PDF-Repräsentation des vollständigen Antragsformulars
        /// - attachment: Anlage, die von einem Bürger hochgeladen wurde
        /// - report: Vom Onlinedienst, nachträglich erzeugte Unterlage
        /// </summary>
        [JsonProperty("purpose")]
        public Purpose Purpose { get; set; }

        [JsonProperty("signature", NullValueHandling = NullValueHandling.Ignore)]
        public AttachmentSignature Signature { get; set; }
    }
    
    /// <summary>
    /// Beschreibt das Signaturformt und Profile
    /// </summary>
    public partial class AttachmentSignature
    {
        /// <summary>
        /// Hier wird die Signatur im Falle einer Detached-Signatur als Base64- oder
        /// Base64Url-kodierte Zeichenkette hinterlegt. Eine Base64Url-Kodierung kommt nur bei
        /// Einsatz von JSON Web Signatures (JWS / JAdES) zum Einsatz.
        /// </summary>
        [JsonProperty("content", NullValueHandling = NullValueHandling.Ignore)]
        public string Content { get; set; }

        /// <summary>
        /// Beschreibt, ob die Signatur als seperate (detached) Signatur (`true`) oder als Teil des
        /// Fachdatensatzes bzw. der Anlage  (`false`) übertragen wird. Wenn der Wert `true` ist,
        /// dann wird die Signatur Base64- oder Base64Url-kodiert im Feld `content` übertragen.
        /// </summary>
        [JsonProperty("detachedSignature")]
        public bool DetachedSignature { get; set; }

        /// <summary>
        /// Referenziert ein eindeutiges Profil einer AdES (advanced electronic signature/seal)
        /// gemäß eIDAS-Verordnung über eine URI gemäß [ETSI TS 119
        /// 192](https://www.etsi.org/deliver/etsi_ts/119100_119199/119192/01.01.01_60/ts_119192v010101p.pdf).
        ///
        /// Für die Details zur Verwendung und Validierung von Profilen siehe auch
        /// https://ec.europa.eu/cefdigital/DSS/webapp-demo/doc/dss-documentation.html#_signatures_profile_simplification
        /// </summary>
        [JsonProperty("eidasAdesProfile", NullValueHandling = NullValueHandling.Ignore)]
        public EidasAdesProfile? EidasAdesProfile { get; set; }

        /// <summary>
        /// Beschreibt, welches Signaturformat die genutzte Signatur / das genutzte Siegel nutzt.
        /// Aktuell wird die Hinterlegung folgender Signaturformate unterstützt: CMS = Cryptographic
        /// Message Syntax, Asic = Associated Signature Containers, PDF = PDF Signatur, XML =
        /// XML-Signature, JSON = JSON Web Signature.
        /// </summary>
        [JsonProperty("signatureFormat")]
        public SignatureFormat SignatureFormat { get; set; }
    }

    /// <summary>
    /// Definiert das Schema und die Signatur(-art), die für die Fachdaten verwendet werden.
    /// </summary>
    public partial class Data
    {
        /// <summary>
        /// Der Hashwert der unverschlüsselten Fachdaten. Die Angabe des Hashwertes dient der
        /// Integritätssicherung des Gesamtantrags und schützt vor einem Austausch der Fachdaten
        /// durch Systeme zwischen Sender und Subscriber (z.B. dem Zustelldienst).
        /// </summary>
        [JsonProperty("hash")]
        public DataHash Hash { get; set; }

        /// <summary>
        /// Beschreibt das Signaturformt und Profile
        /// </summary>
        [JsonProperty("signature", NullValueHandling = NullValueHandling.Ignore)]
        public DataSignature Signature { get; set; }

        /// <summary>
        /// Referenz auf ein Schema, das die Struktur der Fachdaten einer Einreichung beschreibt.
        /// </summary>
        [JsonProperty("submissionSchema")]
        public Fachdatenschema SubmissionSchema { get; set; }
    }

    
    
    /// <summary>
    /// Mimetype (z.B. application/json oder application/xml) des referenzierten Schemas (z.B.
    /// XSD- oder JSON-Schema).
    /// </summary>
    public enum MimeType { ApplicationJson, ApplicationXml };

    /// <summary>
    /// Die vom Benutzer ausgewählte Zahlart. Das Feld ist nur bei einer erfolgreichen Zahlung
    /// vorhanden / befüllt.
    /// </summary>
    public enum PaymentMethod {
        [EnumMember(Value = "CREDITCARD")]
        Creditcard, 
        [EnumMember(Value = "GIROPAY")]
        Giropay, 
        [EnumMember(Value = "INVOICE")]
        Invoice, 
        [EnumMember(Value = "OTHER")]
        Other,
        [EnumMember(Value = "PAYDIRECT")]
        Paydirect, 
        [EnumMember(Value = "PAYPAL")]
        Paypal
    };

    /// <summary>
    /// - INITIAL - der Einreichung hat einen Payment-Request ausgelöst und eine
    /// Payment-Transaction wurde angelegt. Der Nutzer hat aber im Bezahldienst noch keine
    /// Wirkung erzeugt.
    /// - BOOKED - der Nutzer hat die Bezahlung im Bezahldienst autorisiert.
    /// - FAILED - der Vorgang wurde vom Bezahldienst aufgrund der Nutzereingaben abgebrochen.
    /// - CANCELED - der Nutzer hat die Bezahlung im Bezahldienst abgebrochen.
    /// </summary>
    public enum PaymentStatus {
        [EnumMember(Value = "BOOKED")]
        Booked, 
        [EnumMember(Value = "CANCELED")]
        Canceled, 
        [EnumMember(Value = "FAILED")]
        Failed, 
        [EnumMember(Value = "INITIAL")]
        Initial
    };
        

    public partial class Metadata
    {
        public static Metadata FromJson(string json) => JsonConvert.DeserializeObject<Metadata>(json, FitConnect.Models.Api.Set.Converter.Settings);
    }

    public static class Serialize
    {
        public static string ToJson(this Metadata self) => JsonConvert.SerializeObject(self, FitConnect.Models.Api.Set.Converter.Settings);
    }

    internal static class Converter
    {
        public static readonly JsonSerializerSettings Settings = new JsonSerializerSettings
        {
            MetadataPropertyHandling = MetadataPropertyHandling.Ignore,
            DateParseHandling = DateParseHandling.None,
            Converters =
            {
                AuthenticationInformationTypeConverter.Singleton,
                HashTypeConverter.Singleton,
                PurposeConverter.Singleton,
                EidasAdesProfileConverter.Singleton,
                SignatureFormatConverter.Singleton,
                MimeTypeConverter.Singleton,
                PaymentMethodConverter.Singleton,
                StatusConverter.Singleton,
                new IsoDateTimeConverter { DateTimeStyles = DateTimeStyles.AssumeUniversal }
            },
        };
    }

    internal class AuthenticationInformationTypeConverter : JsonConverter
    {
        public override bool CanConvert(Type t) => t == typeof(AuthenticationInformationType) || t == typeof(AuthenticationInformationType?);

        public override object ReadJson(JsonReader reader, Type t, object existingValue, JsonSerializer serializer)
        {
            if (reader.TokenType == JsonToken.Null) return null;
            var value = serializer.Deserialize<string>(reader);
            if (value == "identificationReport")
            {
                return AuthenticationInformationType.IdentificationReport;
            }
            throw new Exception("Cannot unmarshal type AuthenticationInformationType");
        }

        public override void WriteJson(JsonWriter writer, object untypedValue, JsonSerializer serializer)
        {
            if (untypedValue == null)
            {
                serializer.Serialize(writer, null);
                return;
            }
            var value = (AuthenticationInformationType)untypedValue;
            if (value == AuthenticationInformationType.IdentificationReport)
            {
                serializer.Serialize(writer, "identificationReport");
                return;
            }
            throw new Exception("Cannot marshal type AuthenticationInformationType");
        }

        public static readonly AuthenticationInformationTypeConverter Singleton = new AuthenticationInformationTypeConverter();
    }

    internal class HashTypeConverter : JsonConverter
    {
        public override bool CanConvert(Type t) => t == typeof(HashType) || t == typeof(HashType?);

        public override object ReadJson(JsonReader reader, Type t, object existingValue, JsonSerializer serializer)
        {
            if (reader.TokenType == JsonToken.Null) return null;
            var value = serializer.Deserialize<string>(reader);
            if (value == "sha512")
            {
                return HashType.Sha512;
            }
            throw new Exception("Cannot unmarshal type HashType");
        }

        public override void WriteJson(JsonWriter writer, object untypedValue, JsonSerializer serializer)
        {
            if (untypedValue == null)
            {
                serializer.Serialize(writer, null);
                return;
            }
            var value = (HashType)untypedValue;
            if (value == HashType.Sha512)
            {
                serializer.Serialize(writer, "sha512");
                return;
            }
            throw new Exception("Cannot marshal type HashType");
        }

        public static readonly HashTypeConverter Singleton = new HashTypeConverter();
    }

    internal class PurposeConverter : JsonConverter
    {
        public override bool CanConvert(Type t) => t == typeof(Purpose) || t == typeof(Purpose?);

        public override object ReadJson(JsonReader reader, Type t, object existingValue, JsonSerializer serializer)
        {
            if (reader.TokenType == JsonToken.Null) return null;
            var value = serializer.Deserialize<string>(reader);
            switch (value)
            {
                case "attachment":
                    return Purpose.Attachment;
                case "form":
                    return Purpose.Form;
                case "report":
                    return Purpose.Report;
            }
            throw new Exception("Cannot unmarshal type Purpose");
        }

        public override void WriteJson(JsonWriter writer, object untypedValue, JsonSerializer serializer)
        {
            if (untypedValue == null)
            {
                serializer.Serialize(writer, null);
                return;
            }
            var value = (Purpose)untypedValue;
            switch (value)
            {
                case Purpose.Attachment:
                    serializer.Serialize(writer, "attachment");
                    return;
                case Purpose.Form:
                    serializer.Serialize(writer, "form");
                    return;
                case Purpose.Report:
                    serializer.Serialize(writer, "report");
                    return;
            }
            throw new Exception("Cannot marshal type Purpose");
        }

        public static readonly PurposeConverter Singleton = new PurposeConverter();
    }

    internal class EidasAdesProfileConverter : JsonConverter
    {
        public override bool CanConvert(Type t) => t == typeof(EidasAdesProfile) || t == typeof(EidasAdesProfile?);

        public override object ReadJson(JsonReader reader, Type t, object existingValue, JsonSerializer serializer)
        {
            if (reader.TokenType == JsonToken.Null) return null;
            var value = serializer.Deserialize<string>(reader);
            switch (value)
            {
                case "http://uri.etsi.org/ades/191x2/level/baseline/B-B#":
                    return EidasAdesProfile.HttpUriEtsiOrgAdes191X2LevelBaselineBB;
                case "http://uri.etsi.org/ades/191x2/level/baseline/B-LT#":
                    return EidasAdesProfile.HttpUriEtsiOrgAdes191X2LevelBaselineBLt;
                case "http://uri.etsi.org/ades/191x2/level/baseline/B-LTA#":
                    return EidasAdesProfile.HttpUriEtsiOrgAdes191X2LevelBaselineBLta;
                case "http://uri.etsi.org/ades/191x2/level/baseline/B-T#":
                    return EidasAdesProfile.HttpUriEtsiOrgAdes191X2LevelBaselineBT;
            }
            throw new Exception("Cannot unmarshal type EidasAdesProfile");
        }

        public override void WriteJson(JsonWriter writer, object untypedValue, JsonSerializer serializer)
        {
            if (untypedValue == null)
            {
                serializer.Serialize(writer, null);
                return;
            }
            var value = (EidasAdesProfile)untypedValue;
            switch (value)
            {
                case EidasAdesProfile.HttpUriEtsiOrgAdes191X2LevelBaselineBB:
                    serializer.Serialize(writer, "http://uri.etsi.org/ades/191x2/level/baseline/B-B#");
                    return;
                case EidasAdesProfile.HttpUriEtsiOrgAdes191X2LevelBaselineBLt:
                    serializer.Serialize(writer, "http://uri.etsi.org/ades/191x2/level/baseline/B-LT#");
                    return;
                case EidasAdesProfile.HttpUriEtsiOrgAdes191X2LevelBaselineBLta:
                    serializer.Serialize(writer, "http://uri.etsi.org/ades/191x2/level/baseline/B-LTA#");
                    return;
                case EidasAdesProfile.HttpUriEtsiOrgAdes191X2LevelBaselineBT:
                    serializer.Serialize(writer, "http://uri.etsi.org/ades/191x2/level/baseline/B-T#");
                    return;
            }
            throw new Exception("Cannot marshal type EidasAdesProfile");
        }

        public static readonly EidasAdesProfileConverter Singleton = new EidasAdesProfileConverter();
    }

    internal class SignatureFormatConverter : JsonConverter
    {
        public override bool CanConvert(Type t) => t == typeof(SignatureFormat) || t == typeof(SignatureFormat?);

        public override object ReadJson(JsonReader reader, Type t, object existingValue, JsonSerializer serializer)
        {
            if (reader.TokenType == JsonToken.Null) return null;
            var value = serializer.Deserialize<string>(reader);
            switch (value)
            {
                case "asic":
                    return SignatureFormat.Asic;
                case "cms":
                    return SignatureFormat.Cms;
                case "json":
                    return SignatureFormat.Json;
                case "pdf":
                    return SignatureFormat.Pdf;
                case "xml":
                    return SignatureFormat.Xml;
            }
            throw new Exception("Cannot unmarshal type SignatureFormat");
        }

        public override void WriteJson(JsonWriter writer, object untypedValue, JsonSerializer serializer)
        {
            if (untypedValue == null)
            {
                serializer.Serialize(writer, null);
                return;
            }
            var value = (SignatureFormat)untypedValue;
            switch (value)
            {
                case SignatureFormat.Asic:
                    serializer.Serialize(writer, "asic");
                    return;
                case SignatureFormat.Cms:
                    serializer.Serialize(writer, "cms");
                    return;
                case SignatureFormat.Json:
                    serializer.Serialize(writer, "json");
                    return;
                case SignatureFormat.Pdf:
                    serializer.Serialize(writer, "pdf");
                    return;
                case SignatureFormat.Xml:
                    serializer.Serialize(writer, "xml");
                    return;
            }
            throw new Exception("Cannot marshal type SignatureFormat");
        }

        public static readonly SignatureFormatConverter Singleton = new SignatureFormatConverter();
    }

    internal class MimeTypeConverter : JsonConverter
    {
        public override bool CanConvert(Type t) => t == typeof(MimeType) || t == typeof(MimeType?);

        public override object ReadJson(JsonReader reader, Type t, object existingValue, JsonSerializer serializer)
        {
            if (reader.TokenType == JsonToken.Null) return null;
            var value = serializer.Deserialize<string>(reader);
            switch (value)
            {
                case "application/json":
                    return MimeType.ApplicationJson;
                case "application/xml":
                    return MimeType.ApplicationXml;
            }
            throw new Exception("Cannot unmarshal type MimeType");
        }

        public override void WriteJson(JsonWriter writer, object untypedValue, JsonSerializer serializer)
        {
            if (untypedValue == null)
            {
                serializer.Serialize(writer, null);
                return;
            }
            var value = (MimeType)untypedValue;
            switch (value)
            {
                case MimeType.ApplicationJson:
                    serializer.Serialize(writer, "application/json");
                    return;
                case MimeType.ApplicationXml:
                    serializer.Serialize(writer, "application/xml");
                    return;
            }
            throw new Exception("Cannot marshal type MimeType");
        }

        public static readonly MimeTypeConverter Singleton = new MimeTypeConverter();
    }

    internal class MinMaxValueCheckConverter : JsonConverter
    {
        public override bool CanConvert(Type t) => t == typeof(double) || t == typeof(double?);

        public override object ReadJson(JsonReader reader, Type t, object existingValue, JsonSerializer serializer)
        {
            if (reader.TokenType == JsonToken.Null) return null;
            var value = serializer.Deserialize<double>(reader);
            if (value >= 0.01)
            {
                return value;
            }
            throw new Exception("Cannot unmarshal type double");
        }

        public override void WriteJson(JsonWriter writer, object untypedValue, JsonSerializer serializer)
        {
            if (untypedValue == null)
            {
                serializer.Serialize(writer, null);
                return;
            }
            var value = (double)untypedValue;
            if (value >= 0.01)
            {
                serializer.Serialize(writer, value);
                return;
            }
            throw new Exception("Cannot marshal type double");
        }

        public static readonly MinMaxValueCheckConverter Singleton = new MinMaxValueCheckConverter();
    }

    internal class PaymentMethodConverter : JsonConverter
    {
        public override bool CanConvert(Type t) => t == typeof(PaymentMethod) || t == typeof(PaymentMethod?);

        public override object ReadJson(JsonReader reader, Type t, object existingValue, JsonSerializer serializer)
        {
            if (reader.TokenType == JsonToken.Null) return null;
            var value = serializer.Deserialize<string>(reader);
            switch (value)
            {
                case "CREDITCARD":
                    return PaymentMethod.Creditcard;
                case "GIROPAY":
                    return PaymentMethod.Giropay;
                case "INVOICE":
                    return PaymentMethod.Invoice;
                case "OTHER":
                    return PaymentMethod.Other;
                case "PAYDIRECT":
                    return PaymentMethod.Paydirect;
                case "PAYPAL":
                    return PaymentMethod.Paypal;
            }
            throw new Exception("Cannot unmarshal type PaymentMethod");
        }

        public override void WriteJson(JsonWriter writer, object untypedValue, JsonSerializer serializer)
        {
            if (untypedValue == null)
            {
                serializer.Serialize(writer, null);
                return;
            }
            var value = (PaymentMethod)untypedValue;
            switch (value)
            {
                case PaymentMethod.Creditcard:
                    serializer.Serialize(writer, "CREDITCARD");
                    return;
                case PaymentMethod.Giropay:
                    serializer.Serialize(writer, "GIROPAY");
                    return;
                case PaymentMethod.Invoice:
                    serializer.Serialize(writer, "INVOICE");
                    return;
                case PaymentMethod.Other:
                    serializer.Serialize(writer, "OTHER");
                    return;
                case PaymentMethod.Paydirect:
                    serializer.Serialize(writer, "PAYDIRECT");
                    return;
                case PaymentMethod.Paypal:
                    serializer.Serialize(writer, "PAYPAL");
                    return;
            }
            throw new Exception("Cannot marshal type PaymentMethod");
        }

        public static readonly PaymentMethodConverter Singleton = new PaymentMethodConverter();
    }

    internal class PurpleMinMaxLengthCheckConverter : JsonConverter
    {
        public override bool CanConvert(Type t) => t == typeof(string);

        public override object ReadJson(JsonReader reader, Type t, object existingValue, JsonSerializer serializer)
        {
            var value = serializer.Deserialize<string>(reader);
            if (value.Length >= 1 && value.Length <= 36)
            {
                return value;
            }
            throw new Exception("Cannot unmarshal type string");
        }

        public override void WriteJson(JsonWriter writer, object untypedValue, JsonSerializer serializer)
        {
            var value = (string)untypedValue;
            if (value.Length >= 1 && value.Length <= 36)
            {
                serializer.Serialize(writer, value);
                return;
            }
            throw new Exception("Cannot marshal type string");
        }

        public static readonly PurpleMinMaxLengthCheckConverter Singleton = new PurpleMinMaxLengthCheckConverter();
    }

    internal class StatusConverter : JsonConverter
    {
        public override bool CanConvert(Type t) => t == typeof(PaymentStatus) || t == typeof(PaymentStatus?);

        public override object ReadJson(JsonReader reader, Type t, object existingValue, JsonSerializer serializer)
        {
            if (reader.TokenType == JsonToken.Null) return null;
            var value = serializer.Deserialize<string>(reader);
            switch (value)
            {
                case "BOOKED":
                    return PaymentStatus.Booked;
                case "CANCELED":
                    return PaymentStatus.Canceled;
                case "FAILED":
                    return PaymentStatus.Failed;
                case "INITIAL":
                    return PaymentStatus.Initial;
            }
            throw new Exception("Cannot unmarshal type Status");
        }

        public override void WriteJson(JsonWriter writer, object untypedValue, JsonSerializer serializer)
        {
            if (untypedValue == null)
            {
                serializer.Serialize(writer, null);
                return;
            }
            var value = (PaymentStatus)untypedValue;
            switch (value)
            {
                case PaymentStatus.Booked:
                    serializer.Serialize(writer, "BOOKED");
                    return;
                case PaymentStatus.Canceled:
                    serializer.Serialize(writer, "CANCELED");
                    return;
                case PaymentStatus.Failed:
                    serializer.Serialize(writer, "FAILED");
                    return;
                case PaymentStatus.Initial:
                    serializer.Serialize(writer, "INITIAL");
                    return;
            }
            throw new Exception("Cannot marshal type Status");
        }

        public static readonly StatusConverter Singleton = new StatusConverter();
    }

    internal class FluffyMinMaxLengthCheckConverter : JsonConverter
    {
        public override bool CanConvert(Type t) => t == typeof(string);

        public override object ReadJson(JsonReader reader, Type t, object existingValue, JsonSerializer serializer)
        {
            var value = serializer.Deserialize<string>(reader);
            if (value.Length >= 7 && value.Length <= 255)
            {
                return value;
            }
            throw new Exception("Cannot unmarshal type string");
        }

        public override void WriteJson(JsonWriter writer, object untypedValue, JsonSerializer serializer)
        {
            var value = (string)untypedValue;
            if (value.Length >= 7 && value.Length <= 255)
            {
                serializer.Serialize(writer, value);
                return;
            }
            throw new Exception("Cannot marshal type string");
        }

        public static readonly FluffyMinMaxLengthCheckConverter Singleton = new FluffyMinMaxLengthCheckConverter();
    }

    internal class TentacledMinMaxLengthCheckConverter : JsonConverter
    {
        public override bool CanConvert(Type t) => t == typeof(string);

        public override object ReadJson(JsonReader reader, Type t, object existingValue, JsonSerializer serializer)
        {
            var value = serializer.Deserialize<string>(reader);
            if (value.Length <= 10)
            {
                return value;
            }
            throw new Exception("Cannot unmarshal type string");
        }

        public override void WriteJson(JsonWriter writer, object untypedValue, JsonSerializer serializer)
        {
            var value = (string)untypedValue;
            if (value.Length <= 10)
            {
                serializer.Serialize(writer, value);
                return;
            }
            throw new Exception("Cannot marshal type string");
        }

        public static readonly TentacledMinMaxLengthCheckConverter Singleton = new TentacledMinMaxLengthCheckConverter();
    }

    internal class StickyMinMaxLengthCheckConverter : JsonConverter
    {
        public override bool CanConvert(Type t) => t == typeof(string);

        public override object ReadJson(JsonReader reader, Type t, object existingValue, JsonSerializer serializer)
        {
            var value = serializer.Deserialize<string>(reader);
            if (value.Length <= 150)
            {
                return value;
            }
            throw new Exception("Cannot unmarshal type string");
        }

        public override void WriteJson(JsonWriter writer, object untypedValue, JsonSerializer serializer)
        {
            var value = (string)untypedValue;
            if (value.Length <= 150)
            {
                serializer.Serialize(writer, value);
                return;
            }
            throw new Exception("Cannot marshal type string");
        }

        public static readonly StickyMinMaxLengthCheckConverter Singleton = new StickyMinMaxLengthCheckConverter();
    }
}
